CREATE TABLE "KETTLE_JOB"
  (
    "JOB_ID"       VARCHAR2(32) NOT NULL,
    "DIRECTORY" VARCHAR2(255) NOT NULL,
    "NAME"         VARCHAR2(255) NOT NULL,
    "DESCRIPTION" CLOB,
    "EXTENDED_DESCRIPTION" CLOB,
    "STATE"        NUMBER(1) DEFAULT 0 NOT NULL,
    "EXP"          VARCHAR2(50 BYTE) NOT NULL,
    "CREATED_USER"     VARCHAR2(255),
    "CREATED_TIME"     DATE DEFAULT sysdate,
    "MODIFIED_USER"    VARCHAR2(255),
    "MODIFIED_TIME"    DATE,
    "RUN_STATUS"       NUMBER(1),
    "LAST_UPDATE"      DATE,
    "AUTO_RESTART_NUM" NUMBER(2) DEFAULT 0,
    PRIMARY KEY ("JOB_ID")
  );
COMMENT ON COLUMN "KETTLE_JOB"."JOB_ID" IS '主键';
COMMENT ON COLUMN "KETTLE_JOB"."DIRECTORY" IS '目录，相对于资料库根目录的绝对路径';
COMMENT ON COLUMN "KETTLE_JOB"."NAME" IS '作业名称';
COMMENT ON COLUMN "KETTLE_JOB"."DESCRIPTION" IS '作业描述';
COMMENT ON COLUMN "KETTLE_JOB"."EXTENDED_DESCRIPTION" IS '作业扩展描述';
COMMENT ON COLUMN "KETTLE_JOB"."STATE" IS '状态：0=停止，1=启动，2=删除';
COMMENT ON COLUMN "KETTLE_JOB"."EXP" IS '表达式';
COMMENT ON COLUMN "KETTLE_JOB"."CREATED_USER" IS '创建人';
COMMENT ON COLUMN "KETTLE_JOB"."CREATED_TIME" IS '创建日期时间';
COMMENT ON COLUMN "KETTLE_JOB"."MODIFIED_USER" IS '修改人';
COMMENT ON COLUMN "KETTLE_JOB"."MODIFIED_TIME" IS '修改日期时间';
COMMENT ON COLUMN "KETTLE_JOB"."RUN_STATUS" IS '最后运行状态，0：正常结束，1：运行中，2：异常结束';
COMMENT ON COLUMN "KETTLE_JOB"."LAST_UPDATE" IS '最后更新时间';
COMMENT ON COLUMN "KETTLE_JOB"."AUTO_RESTART_NUM" IS '自动重启次数';


CREATE TABLE KETTLE_JOB_PARAMS 
(
    "PARAM_ID"    VARCHAR2(32) NOT NULL,
    "JOB_ID"      VARCHAR2(32) NOT NULL,
    "PARAM_CODE"  VARCHAR2(128) NOT NULL,
    "PARAM_VALUE" VARCHAR2(512),
    "DESCRIPTION" CLOB,
    "EXTENDED_DESCRIPTION" CLOB,
    "STATE"         NUMBER(1,0) NOT NULL,
    "CREATED_USER"   VARCHAR2(255),
    "CREATED_TIME"   DATE DEFAULT sysdate,
    "MODIFIED_USER" VARCHAR2(255),
    "MODIFIED_TIME" DATE,
    PRIMARY KEY ("PARAM_ID")
);
COMMENT ON TABLE "KETTLE"."KETTLE_JOB_PARAMS" IS '任务默认参数配置表';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."PARAM_ID" IS '参数ID，主键';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."JOB_ID" IS '任务ID，对应KETTLE_JOB表的JOB_ID';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."PARAM_CODE" IS '参数名';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."PARAM_VALUE" IS '参数值';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."DESCRIPTION" IS '参数描述';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."EXTENDED_DESCRIPTION" IS '参数描述扩展';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."STATE" IS '参数状态：0=禁用：1=启用，2=删除';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."CREATED_USER" IS '创建人';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."CREATED_TIME" IS '创建日期时间';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."MODIFIED_USER" IS '修改人';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_PARAMS"."MODIFIED_TIME" IS '修改日期时间';

/*ALTER TABLE KETTLE_JOB_PARAMS ADD CONSTRAINT KETTLE_JOB_PARAMS_UK1 UNIQUE 
(
  JOB_ID,
  PARAM_CODE 
);*/

ALTER TABLE KETTLE_JOB_PARAMS ADD CONSTRAINT KETTLE_JOB_PARAMS_FK1 FOREIGN KEY
(
  JOB_ID 
)
REFERENCES KETTLE_JOB
(
  JOB_ID 
)
ON DELETE CASCADE;


CREATE TABLE "KETTLE"."KETTLE_JOB_RUN_LOG"
(
  "LOG_ID"     VARCHAR2(32) NOT NULL,
  "JOB_ID"     VARCHAR2(32) NOT NULL,
  "START_TIME" DATE NOT NULL,
  "END_TIME"   DATE NOT NULL,
  "RESULT"     VARCHAR2(200) NOT NULL,
  "LOG_FILE"   VARCHAR2(1000),
  PRIMARY KEY ("LOG_ID")
);
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_RUN_LOG"."LOG_ID" IS '运行日志ID';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_RUN_LOG"."JOB_ID" IS '任务ID';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_RUN_LOG"."START_TIME" IS '开始时间';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_RUN_LOG"."END_TIME" IS '结束时间';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_RUN_LOG"."RESULT" IS '运行结果';
COMMENT ON COLUMN "KETTLE"."KETTLE_JOB_RUN_LOG"."LOG_FILE" IS '日志文件';
COMMENT ON TABLE "KETTLE"."KETTLE_JOB_RUN_LOG" IS 'Kettle任务运行记录表';
